from ast import literal_eval
from os import environ
from types import SimpleNamespace
from typing import Any
from typing import Callable
from typing import Dict
from typing import List
from typing import Optional

from .util import remove_prefix
from .util import safe_execute

NEWLINE = "\n"


class ValidationError(ValueError):
    def __init__(self, var_name: str, hint: Optional[str]):
        self.var_name = var_name
        self.hint = hint

        super().__init__(
            f"contents of var '{var_name}' invalid"
            f"{f' (hint: {hint})' if hint is not None else ''}"
        )


class MissingEnvVarError(ValueError):
    def __init__(self, vars_: List[str]):
        self.vars_ = vars_
        super().__init__(
            f"env variable{'s' if len(vars_) > 1 else ''} missing: {', '.join(vars_)}"
        )


class TransformationError(ValueError):
    def __init__(self, var_name: str, hint: Optional[str], exception: Exception):
        self.var_name = var_name
        self.hint = hint
        self.exception = exception

        super().__init__(
            f"could not transform '{var_name}': {str(exception)} "
            f"(hint: {hint if hint is not None else ''})"
        )


class InvalidDefault:
    pass


INVALID_DEFAULT = InvalidDefault()


class Default:
    def __init__(
        self,
        value: Any,
        validator: Optional[Callable[[Any], bool]] = None,
        transformer: Optional[Callable[[Any], Any]] = None,
        hint: Optional[str] = None,
    ):
        self.value = value
        self.validator = validator
        self.transformer = transformer
        self.hint = hint

    def validate(self, value):
        return (
            False
            if value is INVALID_DEFAULT
            else self.validator(value)
            if callable(self.validator)
            else True
        )

    def transform(self, value):
        return self.transformer(value) if callable(self.transformer) else value


def check_prefix(env_var_prefix: str) -> str:
    return env_var_prefix if env_var_prefix[-1] == "_" else f"{env_var_prefix}_"


def template(env_var_prefix: str, defaults: Dict[str, Default]) -> Optional[str]:
    template = ""

    env_var_prefix = check_prefix(env_var_prefix)
    for name in defaults:
        default = defaults[name]
        template = (
            f"{template}{f'# {default.hint}{NEWLINE}' if default.hint is not None else ''}"
            f"export {env_var_prefix}{name}="
            f"{default.value if default.value not in [None, InvalidDefault] else ''}\n"
        )

    return template


def build_env(env_var_prefix: str, defaults: Dict[str, Default]):
    # Make sure the environment variable prefix ends with a underscore ('_')
    env_var_prefix = check_prefix(env_var_prefix)
    env = SimpleNamespace()

    for var in defaults:
        value = defaults[var].value
        prefix_less = remove_prefix(var, env_var_prefix)
        if defaults[prefix_less].validate(value):
            # Transformations are not applied for default values
            setattr(env, var.lower(), value)
        else:
            env_var = f"{env_var_prefix}{var}"
            # Invalid default might be substituted later
            if env_var not in environ:
                raise ValidationError(env_var, defaults[prefix_less].hint)
    for var in environ:
        if var.startswith(env_var_prefix):
            prefix_less = remove_prefix(var, env_var_prefix)
            if prefix_less in defaults:
                # Safely evaluate value
                value = safe_execute(
                    environ[var],
                    (ValueError, SyntaxError),
                    lambda: literal_eval(environ[var]),
                )
                # Apply transformation
                try:
                    transformed_value = defaults[prefix_less].transform(value)
                except Exception as e:
                    raise TransformationError(
                        prefix_less, defaults[prefix_less].hint, e
                    )
                if defaults[prefix_less].validate(transformed_value):
                    setattr(env, prefix_less.lower(), transformed_value)
                else:
                    raise ValidationError(var, defaults[prefix_less].hint)
            else:
                # To-Do: User feedback
                pass

    missing_vars = []

    for var in defaults:
        if var.lower() not in env.__dict__:
            missing_vars.append(var)

    if len(missing_vars) > 0:
        raise MissingEnvVarError(missing_vars)

    return env



import sys
from io import BytesIO
from os.path import isfile
from platform import mac_ver
from subprocess import call

from AppKit import NSBitmapImageRep
from AppKit import NSData
from AppKit import NSImage
from AppKit import NSMakeSize
from Foundation import NSUserNotification
from Foundation import NSUserNotificationCenter
from mutagen import File
from PIL import Image

from cmus_osx.constants import CMUS_OSX_FOLDER_NAME
from cmus_osx.constants import CONFIG_NAME
from cmus_osx.constants import ENV
from cmus_osx.constants import ENV_VAR_PREFIX
from cmus_osx.env import build_env
from cmus_osx.util import locate_cmus_base_path
from cmus_osx.util import source_env_file


def exception_hook(exc_type, exc_value, exc_traceback):
    call(["cmus-remote", "--raw", "echo cmus-osx error: %s" % str(exc_value)])


sys.excepthook = exception_hook

cmus_base_path = locate_cmus_base_path()

if cmus_base_path is not None:
    source_env_file(cmus_base_path / CMUS_OSX_FOLDER_NAME / CONFIG_NAME)

# Use defaults values if config file can't be located
env = build_env(ENV_VAR_PREFIX, ENV)

status_raw = sys.argv[1:]
status = dict(zip(status_raw[0::2], status_raw[1::2]))

# Quickly exit if paused
if "status" in status:
    if not env.notification_on_pause:
        if status["status"] != "playing":
            exit(0)

cover = None

if "url" in status:
    status["status"] = "(streaming ...)"
    # the title may contain both the artist and the song name
    if "title" in status:
        title_pair = status["title"].split(" - ")
        if len(title_pair) > 1:
            status["artist"] = title_pair[0]
            status["title"] = title_pair[1]
    else:
        status["title"] = status["url"]
elif "file" in status and isfile(status["file"]):
    file = File(status["file"])
    # id3
    if "APIC:" in file:
        cover = file["APIC:"]
        cover = cover.data
    # mp4
    elif "covr" in file:
        covers = file["covr"]
        if len(covers) > 0:
            cover = covers[0]
    # flac
    elif hasattr(file, "pictures") and len(file.pictures) > 0:
        cover = file.pictures[0].data

if env.notification_on_pause:
    title = "cmus %s" % status["status"]
else:
    title = "cmus"
subtitle = ""
message = ""

if "tracknumber" in status and status["tracknumber"].isnumeric():
    subtitle += "%s. " % status["tracknumber"]

if "title" in status:
    subtitle += status["title"]

if "artist" in status:
    message += status["artist"]
elif "albumartist" in status:
    message += status["albumartist"]

if "album" in status:
    message += " – %s" % status["album"]

if "date" in status and status["date"].isnumeric():
    message += " (%s)" % status["date"]


center = NSUserNotificationCenter.defaultUserNotificationCenter()
notification = NSUserNotification.alloc().init()

notification.setTitle_(title)
notification.setSubtitle_(subtitle)
notification.setInformativeText_(message)

# To-Do: Data allocation currently doesn't work in Catalina
if mac_ver()[0] != "10.15":
    if cover is not None:  # the song has an embedded cover image
        data = NSData.alloc().initWithBytes_length_(cover, len(cover))
        image_rep = NSBitmapImageRep.alloc().initWithData_(data)

        # CGImageGetWidth started returning bogus values in macOS 10.14 ->
        # Use Pillow to extract the image dimensions
        size = NSMakeSize(*Image.open(BytesIO(cover)).size)

        image = NSImage.alloc().initWithSize_(size)
        image.addRepresentation_(image_rep)
        if env.itunes_style_notification:

            notification.setValue_forKey_(image, "_identityImage")
        else:
            notification.setValue_forKey_(
                NSImage.alloc().initByReferencingFile_(str(env.app_icon)),
                "_identityImage",
            )
            notification.setContentImage_(image)
    else:  # song has no cover image, show an icon
        notification.setValue_forKey_(
            NSImage.alloc().initByReferencingFile_(str(env.app_icon)), "_identityImage"
        )

center.removeAllDeliveredNotifications()

center.deliverNotification_(notification)



from os import _exit as exit
from subprocess import call as call_
from subprocess import CalledProcessError
from subprocess import check_output
from threading import Thread
from time import sleep
from typing import List
from typing import Optional

from AppKit import NSApplication
from AppKit import NSApplicationActivationPolicyProhibited
from AppKit import NSKeyUp
from AppKit import NSSystemDefined
from PyObjCTools import AppHelper

from cmus_osx.constants import CMUS_OSX_FOLDER_NAME
from cmus_osx.constants import CONFIG_NAME
from cmus_osx.constants import ENV
from cmus_osx.constants import ENV_VAR_PREFIX
from cmus_osx.env import build_env
from cmus_osx.util import locate_cmus_base_path
from cmus_osx.util import source_env_file
from cmus_osx.util import throttle

cmus_base_path = locate_cmus_base_path()

if cmus_base_path is not None:
    source_env_file(cmus_base_path / CMUS_OSX_FOLDER_NAME / CONFIG_NAME)

# Use defaults values if config file can't be located
env = build_env(ENV_VAR_PREFIX, ENV)

call = throttle(env.throttle_interval)(call_)


class KeySocketApp(NSApplication):
    repeated = False

    def sendEvent_(self, event):
        if event.type() is NSSystemDefined and event.subtype() == 8:
            data = event.data1()
            key_code = (data & 0xFFFF0000) >> 16
            key_flags = data & 0x0000FFFF
            key_state = (key_flags & 0xFF00) >> 8
            key_repeat = key_flags & 0x1

        if key_repeat and key_state is not NSKeyUp:
            if key_code == 20:
                self.repeated = True
                call(["cmus-remote", "-k", "-10"])
            elif key_code == 19:
                self.repeated = True
                call(["cmus-remote", "-k", "+10"])

        if key_state is NSKeyUp:
            if self.repeated:
                self.repeated = False
        elif key_code in (20, 18):
            call(["cmus-remote", "-r"])
        elif key_code == 16:
            call(["cmus-remote", "-u"])
        elif key_code in (19, 17):
            call(["cmus-remote", "-n"])


class SingleInstanceChecker(Thread):
    def __init__(self):
        Thread.__init__(self)
        self.daemon = True
        self.running = False

    def run(self):
        self.running = True
        while self.running:
            cmus_instances = get_cmus_instances()
            if cmus_instances is not None:
                if len(cmus_instances) == 0:
                    exit(0)
                elif len(cmus_instances) > 1:
                    call(
                        [
                            "cmus-remote",
                            "--raw",
                            "echo Media key support disabled "
                            "because more than one cmus instance is running.",
                        ]
                    )
                    exit(1)
            else:
                call(
                    [
                        "cmus-remote",
                        "--raw",
                        "echo Media key support disabled because "
                        "enumerating cmus instances failed.",
                    ]
                )
                exit(1)
            sleep(1)

    def stop(self):
        self.running = False


def get_cmus_instances() -> Optional[List[int]]:
    try:
        return [
            int(pid)
            for pid in check_output(["pgrep", "-x", "cmus"]).decode().split("\n")
            if pid != ""
        ]
    except CalledProcessError:
        return None


cmus_instances = get_cmus_instances()
if cmus_instances is not None and len(cmus_instances) == 1:
    app = KeySocketApp.sharedApplication()
    app.setActivationPolicy_(NSApplicationActivationPolicyProhibited)
    single_instance_checker = SingleInstanceChecker()
    single_instance_checker.start()
    AppHelper.runEventLoop()
else:
    exit(1)



from dataclasses import dataclass
from os import chmod
from os import kill
from os import remove
from os import rename
from pathlib import Path
from shutil import rmtree
from signal import SIGTERM
from subprocess import call
from tempfile import NamedTemporaryFile
from typing import Optional

import click
from click import echo
from click import style

from .constants import CMUS_OSX_FOLDER_NAME
from .constants import CONFIG_NAME
from .constants import COULD_NOT_LOCATED_CMUS_DIRECTORY
from .constants import ENV
from .constants import ENV_VAR_PREFIX
from .constants import RC_ENTRY_REGEX
from .constants import RC_SCRIPT_NAME
from .constants import SCRIPTS
from .constants import SDP_SCRIPT_NAME
from .constants import STATUS_DISPLAY_PROGRAM_REGEX
from .env import template
from .util import get_cmus_instances
from .util import locate_cmus_base_path
from .util import locate_editor


class CmusConfig:
    @dataclass
    class _CmusConfig:
        base_path: Path
        # File that holds cmus commands that are executed on startup
        rc_path: Path
        # cmus config file
        autosave_path: Path

    def __new__(self) -> Optional["CmusConfig._CmusConfig"]:
        base_path = locate_cmus_base_path()
        if base_path is not None:
            return CmusConfig._CmusConfig(
                base_path, base_path / "rc", base_path / "autosave"
            )
        else:
            return None


@click.group()
@click.pass_context
def entrypoint(ctx):
    # Entrypoint is called directly
    ctx.ensure_object(dict)

    cmus_config = CmusConfig()

    if cmus_config is not None:
        cmus_osx_base_path = cmus_config.base_path / CMUS_OSX_FOLDER_NAME
        cmus_osx_base_path.mkdir(exist_ok=True)

        config_path = cmus_osx_base_path / CONFIG_NAME

        if not config_path.is_file():
            with open(config_path, "w") as f:
                f.write(template(ENV_VAR_PREFIX, ENV))

        locals_ = locals()

        for local in (local for local in locals_ if local != "ctx"):
            ctx.obj[local] = locals_[local]
    else:
        echo(f"{style('ERROR', fg='red')}: Could not locate cmus config directory")

        exit(COULD_NOT_LOCATED_CMUS_DIRECTORY)


@entrypoint.command()
@click.pass_context
def install(ctx):
    cmus_config = ctx.obj["cmus_config"]
    cmus_osx_base_path = ctx.obj["cmus_osx_base_path"]

    cmus_osx_base_path.mkdir(exist_ok=True)

    for script_name in SCRIPTS:
        script_path = cmus_osx_base_path / script_name
        with open(script_path, "w") as f:
            f.write(f"{SCRIPTS[script_name]}\n")
        chmod(script_path, 0o744)

    rc_script_path = cmus_osx_base_path / RC_SCRIPT_NAME
    sdp_script_path = cmus_osx_base_path / SDP_SCRIPT_NAME

    write_rc = True

    tmp_rc_file = NamedTemporaryFile("w", delete=False)
    with open(cmus_config.rc_path, "r") as f:
        for line in f:
            match = RC_ENTRY_REGEX.match(line)
            # Found invocation of 'rc' script
            if match is not None and Path(match.group(1)) == rc_script_path:
                write_rc = False
            tmp_rc_file.write(line)

    if write_rc:
        tmp_rc_file.write(f"shell {rc_script_path} &\n")
        rename(tmp_rc_file.name, cmus_config.rc_path)
    else:
        remove(tmp_rc_file.name)

    write_autosave = False

    tmp_autosave_file = NamedTemporaryFile("w", delete=False)
    with open(cmus_config.autosave_path, "r") as f:
        for line in f:
            match = STATUS_DISPLAY_PROGRAM_REGEX.match(line)
            if match is not None:
                sdp_value = match.group(1)
                # 'status_display_program' is not set
                if sdp_value == "":
                    # Write 'status_display_program' without asking for permission
                    write_autosave = True
                elif Path(sdp_value) != sdp_script_path:
                    # Ask for permission
                    if click.confirm(
                        f"{style('WARNING', fg='yellow')}: "
                        f"'status_display_program' currently set to '{sdp_value}', "
                        "override?"
                    ):
                        write_autosave = True
                    else:
                        echo(
                            f"{style('WARNING', fg='yellow')}: Manually set "
                            f"'status_display_program' to '{str(sdp_script_path)}'"
                        )
                if write_autosave:
                    tmp_autosave_file.write(
                        f"set status_display_program={str(sdp_script_path)}\n"
                    )
            else:
                tmp_autosave_file.write(line)

    # No need to replace 'autosave' if no changes were written
    if write_autosave:
        rename(tmp_autosave_file.name, cmus_config.autosave_path)
    else:
        remove(tmp_autosave_file.name)

    if not write_rc and not write_autosave:
        echo(f"{style('NOTE', fg='magenta')}: Already installed.")
    else:
        echo(f"{style('SUCCESS', fg='green')}: Successfully installed.")


@entrypoint.command()
@click.pass_context
def uninstall(ctx):
    cmus_config = ctx.obj["cmus_config"]
    cmus_osx_base_path = ctx.obj["cmus_osx_base_path"]

    try:
        rmtree(cmus_osx_base_path)
    except FileNotFoundError:
        pass

    cmus_instances = get_cmus_instances()

    if cmus_instances is not None:
        for pid in cmus_instances:
            kill(pid, SIGTERM)

    rc_script_path = cmus_osx_base_path / RC_SCRIPT_NAME
    sdp_script_path = cmus_osx_base_path / SDP_SCRIPT_NAME

    write_rc = False

    tmp_rc_file = NamedTemporaryFile("w", delete=False)
    with open(cmus_config.rc_path, "r") as f:
        for line in f:
            match = RC_ENTRY_REGEX.match(line)
            # Exclude invocations of 'rc' script
            if not (match is not None and Path(match.group(1)) == rc_script_path):
                tmp_rc_file.write(line)
            else:
                write_rc = True

    if write_rc:
        rename(tmp_rc_file.name, cmus_config.rc_path)
    else:
        remove(tmp_rc_file.name)

    write_autosave = False

    tmp_autosave_file = NamedTemporaryFile("w", delete=False)
    with open(cmus_config.autosave_path, "r") as f:
        for line in f:
            match = STATUS_DISPLAY_PROGRAM_REGEX.match(line)
            # Exclude invocations of 'status_display_program' script
            if not (match is not None and Path(match.group(1)) == sdp_script_path):
                tmp_autosave_file.write(line)
            else:
                tmp_autosave_file.write("set status_display_program=\n")
                write_autosave = True

    # No need to replace 'autosave' if no changes were written
    if write_autosave:
        rename(tmp_autosave_file.name, cmus_config.autosave_path)
    else:
        remove(tmp_autosave_file.name)

    if not write_rc and not write_autosave:
        echo(f"{style('NOTE', fg='magenta')}: Already uninstalled.")
    else:
        echo(f"{style('SUCCESS', fg='green')}: Successfully uninstalled.")


@entrypoint.command()
@click.pass_context
def config(ctx):
    cmus_osx_base_path = ctx.obj["cmus_osx_base_path"]

    config_path = cmus_osx_base_path / CONFIG_NAME

    editor = locate_editor()

    if editor is None:
        echo(
            f"{style('ERROR', fg='red')}: Could not locate editor, "
            f"manually edit '{str(config_path)}'"
        )
    else:
        call([str(editor), str(config_path)])



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['cmus_osx']

package_data = \
{'': ['*'], 'cmus_osx': ['payload/*', 'resource/*']}

install_requires = \
['Pillow>=6.1.0,<7.0.0',
 'click>=7.0,<8.0',
 'mutagen>=1.42,<2.0',
 'pyobjc>=5.2,<6.0']

entry_points = \
{'console_scripts': ['cmus-osx = cmus_osx:entrypoint']}

setup_kwargs = {
    'name': 'cmus-osx',
    'version': '3.0.4',
    'description': 'Adds track change notifications, and media key support to cmus.',
    'long_description': '<img align="right" src="https://user-images.githubusercontent.com/9287847/33808557-f03eef40-dde8-11e7-8951-68350df85a70.gif" width="350"/>\n\n<p>\xe2\x80\x8b</p>\n\n<h1><kbd>\xe2\x96\xb6</kbd> cmus-osx</h1>\n\n![Python version support: 3.7](https://img.shields.io/badge/python-3.7-limegreen.svg)\n[![PyPI version](https://badge.fury.io/py/cmus-osx.svg)](https://badge.fury.io/py/cmus-osx)\n![License: MIT](https://img.shields.io/badge/license-MIT-limegreen.svg)\n[![CircleCI](https://circleci.com/gh/PhilipTrauner/cmus-osx.svg?style=svg)](https://circleci.com/gh/PhilipTrauner/cmus-osx)\n\n**cmus-osx** adds track change notifications, and media key support to [*cmus*](https://cmus.github.io/) (*macOS* only).\n\n## Installation\nmacOS automatically launches iTunes on media key presses.\nInstalling [noTunes](https://github.com/tombonez/noTunes/releases/tag/v2.0) is the recommended solution to prevent this from happening.\n\n```bash\npip3 install cmus-osx\ncmus-osx install\n```\n\n**cmus-osx** supports virtual environments natively, so installing it via `pipx` (or basically any other virtual environment manager) works just as well.\n\n### Uninstall\n```\ncmus-osx uninstall\npip3 uninstall cmus-osx\n```\n\n#### pyenv\nFramework building has to be enabled, otherwise notifications cannot be created.\n```bash\nexport PYTHON_CONFIGURE_OPTS="--enable-framework"\n```\n\n## Configuration\n```\ncmus-osx config\n```\n\n### Credits\n* [azadkuh](https://github.com/azadkuh): all versions up to and including v1.2.0\n* [PhilipTrauner](https://github.com/PhilipTrauner): all following versions\n',
    'author': 'Philip Trauner',
    'author_email': 'philip.trauner@arztpraxis.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from os import environ
from os import getenv
from pathlib import Path
from shutil import which
from subprocess import CalledProcessError
from subprocess import check_output
from subprocess import PIPE
from subprocess import Popen
from time import time
from typing import Any
from typing import Callable
from typing import List
from typing import Optional
from typing import Sequence
from typing import Type
from typing import Union


def locate_cmus_base_path() -> Optional[Path]:
    for path in (
        path.expanduser() for path in (Path("~/.config/cmus/"), Path("~/.cmus/"))
    ):
        if path.is_dir():
            return path
    return None


def locate_editor() -> Optional[Path]:
    for editor in (getenv("VISUAL", None), getenv("EDITOR", None), "nano", "vim", "vi"):
        if editor is not None:
            # Might be absolute path to editor
            editor_path = Path(editor).expanduser()
            # Might also be just the binary name
            editor_which = which(editor)

            if editor_path.is_file():
                return editor_path
            elif editor_which is not None:
                return Path(editor_which)
    return None


def safe_execute(
    default: Any,
    exception: Union[Type[BaseException], Sequence[Type[BaseException]]],
    function: Callable,
    *args: Any,
    **kwargs: Any,
):
    try:
        return function(*args, **kwargs)
    except exception:  # type: ignore
        return default


def remove_prefix(text: str, prefix: str):
    return text[len(prefix) :] if text.startswith(prefix) else text


# https://stackoverflow.com/a/3505826
def source_env_file(env_file: Path):
    proc = Popen(
        ["/bin/sh", "-c", f"set -a && source {str(env_file)} && env"], stdout=PIPE
    )
    for line in (line.decode() for line in proc.stdout):
        (key, _, value) = line.partition("=")
        environ[key] = value.rstrip()
    proc.communicate()


def get_cmus_instances() -> Optional[List[int]]:
    try:
        return [
            int(pid)
            for pid in check_output(["pgrep", "-x", "cmus"]).decode().split("\n")
            if pid != ""
        ]
    except CalledProcessError:
        return None


# https://gist.github.com/walkermatt/2871026#gistcomment-2280711
def throttle(interval: Union[float, int]):
    """Decorator ensures function that can only be called once every `s` seconds.
    """

    def decorate(fn: Callable) -> Callable:
        t = None

        def wrapped(*args, **kwargs):
            nonlocal t
            t_ = time()
            if t is None or t_ - t >= interval:
                result = fn(*args, **kwargs)
                t = time()
                return result

        return wrapped

    return decorate



from pathlib import Path
from re import compile as re_compile
from sys import executable
from sys import modules
from typing import Dict

from .env import Default

STATUS_DISPLAY_PROGRAM_REGEX = re_compile(r"set status_display_program=(.*)")
RC_ENTRY_REGEX = re_compile(r"shell\s(.*)\s&")

INTERPRETER_PATH = Path(executable)
MODULE_BASE_PATH = Path(modules[__name__].__file__).parent

RESOURCE_PATH = MODULE_BASE_PATH / "resource"
PAYLOAD_PATH = MODULE_BASE_PATH / "payload"

APP_ICON_NAME = "icon.png"
RC_SCRIPT_NAME = "rc_entry.sh"
SDP_SCRIPT_NAME = "status_display_program.sh"

APP_ICON_PATH = RESOURCE_PATH / APP_ICON_NAME

RC_PATH = PAYLOAD_PATH / "media_keys.py"
# (s)tatus_(d)isplay_(p)rogram
SDP_PATH = PAYLOAD_PATH / "notify.py"

CMUS_OSX_FOLDER_NAME = "cmus-osx"
CONFIG_NAME = "cmus-osx.conf"

SCRIPTS: Dict[str, str] = {
    RC_SCRIPT_NAME: f"#!/bin/sh\n\n{str(INTERPRETER_PATH)} {str(RC_PATH)}",
    SDP_SCRIPT_NAME: f'#!/bin/sh\n\n{str(INTERPRETER_PATH)} {str(SDP_PATH)} "${{@}}"',
}

COULD_NOT_LOCATED_CMUS_DIRECTORY = 1

ENV_VAR_PREFIX = "CMUS_OSX"
ENV: Dict[str, Default] = {
    "NOTIFICATION_ON_PAUSE": Default(
        False,
        validator=lambda value: type(value) is bool,
        hint="Controls if a notification should be display on pause",
    ),
    "ITUNES_STYLE_NOTIFICATION": Default(
        True,
        validator=lambda value: type(value) is bool,
        hint="Display album artwork as app icon instead of notification badge",
    ),
    "APP_ICON": Default(
        APP_ICON_PATH,
        validator=lambda value: value.expanduser().is_file(),
        transformer=lambda value: Path(value),
        hint="Fallback icon if album artwork extraction fails",
    ),
    "THROTTLE_INTERVAL": Default(
        0.0,
        validator=lambda value: value >= 0,
        hint="Throttle interval (adjust if experiencing duplicated key-pressses)",
    ),
}
